// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Copyright 2017 and onwards Google, Inc.

#ifndef BAUMWELCH_TRAINSCRIPT_H_
#define BAUMWELCH_TRAINSCRIPT_H_

#include <fst/extensions/far/far-class.h>
#include <fst/script/fst-class.h>
#include <baumwelch/getters.h>
#include <baumwelch/train.h>

namespace fst {
namespace script {

// 1: Pair construction.
using TrainBaumWelchArgs1 =
    std::tuple<FarReaderClass *, FarReaderClass *, MutableFstClass *,
               ExpectationTableType, const TrainBaumWelchOptions &>;

template <class Arc>
void TrainBaumWelch(TrainBaumWelchArgs1 *args) {
  FarReader<Arc> *plaintext = std::get<0>(*args)->GetFarReader<Arc>();
  FarReader<Arc> *ciphertext = std::get<1>(*args)->GetFarReader<Arc>();
  MutableFst<Arc> *channel = std::get<2>(*args)->GetMutableFst<Arc>();
  TrainBaumWelch(plaintext, ciphertext, channel, std::get<3>(*args),
                 std::get<4>(*args));
}

// 2: Decipherment construction.
using TrainBaumWelchArgs2 =
    std::tuple<const FstClass &, FarReaderClass *, MutableFstClass *,
               ExpectationTableType, const TrainBaumWelchOptions &>;

template <class Arc>
void TrainBaumWelch(TrainBaumWelchArgs2 *args) {
  const Fst<Arc> &plaintext = *(std::get<0>(*args).GetFst<Arc>());
  FarReader<Arc> *ciphertext = std::get<1>(*args)->GetFarReader<Arc>();
  MutableFst<Arc> *channel = std::get<2>(*args)->GetMutableFst<Arc>();
  TrainBaumWelch(plaintext, ciphertext, channel, std::get<3>(*args),
                 std::get<4>(*args));
}

void TrainBaumWelch(
    FarReaderClass *plaintext, FarReaderClass *ciphertext,
    MutableFstClass *channel, ExpectationTableType etype = STATE_ILABEL,
    const TrainBaumWelchOptions &opts = TrainBaumWelchOptions());

void TrainBaumWelch(
    const FstClass &plaintext, FarReaderClass *ciphertext,
    MutableFstClass *channel, ExpectationTableType etype = STATE_ILABEL,
    const TrainBaumWelchOptions &opts = TrainBaumWelchOptions());

}  // namespace script
}  // namespace fst

#endif  // BAUMWELCH_TRAINSCRIPT_H_

